import os
import sys
import subprocess
from dotenv import load_dotenv
from pathlib import Path

# For accessing package resources
try:
    from importlib.resources import path as resources_path
except ImportError:
    # For Python versions < 3.7
    from importlib_resources import path as resources_path  # type: ignore


def write_env(key, value, env_file=".env"):
    """
    Write or update key-value pairs in a .env file in the current working directory.
    """
    env_path = os.path.join(current_dir, env_file)
    if not os.path.exists(env_path):
        with open(env_path, "w") as f:
            f.write(f"{key}={value}\n")
    else:
        replaced = False
        with open(env_path, "r") as f:
            lines = f.readlines()
        with open(env_path, "w") as f:
            for line in lines:
                if line.startswith(f"{key}="):
                    f.write(f"{key}={value}\n")
                    replaced = True
                else:
                    f.write(line)
            if not replaced:
                f.write(f"{key}={value}\n")


def prompt(question, default_value=None):
    """
    Prompt user for input with an optional default value.
    """
    if default_value:
        question = f"{question} (default value: {default_value}) "
    else:
        question = f"{question} "
    user_input = input(question).strip()
    if not user_input and default_value is not None:
        return default_value
    return user_input


def main():
    global current_dir
    # Set current directory
    current_dir = os.getcwd()
    dotenv_path = os.path.join(current_dir, ".env")
    load_dotenv(dotenv_path=dotenv_path)
    PROJECT_NAME = os.getenv("PROJECT_NAME")
    BLOCKCHAIN_NETWORK = os.getenv("BLOCKCHAIN_NETWORK")
    PRIVATE_KEY = os.getenv("PRIVATE_KEY")
    DEVELOPER_FEE = os.getenv("DEVELOPER_FEE")
    SERVICE_TYPE = os.getenv("SERVICE_TYPE")
    VERSION = os.getenv("VERSION")

    imageRegistryPy_path = (
        Path(__file__).resolve().parent / "pynithy" / "run" / "image_registry.py"
    )
    result = ""

    import ethernity_cloud_sdk_py.commands.pynithy.run.image_registry as image_registry

    if not (PROJECT_NAME and BLOCKCHAIN_NETWORK and PRIVATE_KEY and DEVELOPER_FEE):
        has_wallet = prompt(
            "Do you have an existing wallet?", default_value="yes"
        ).lower()
        print()
        if has_wallet != "yes":
            print("Without a wallet, you will not be able to publish.")
            print(
                "Please refer to Blockchain Wallets Documentation (https://docs.ethernity.cloud/ethernity-node/prerequisites-ethernity-node/blockchain-wallets)."
            )
            sys.exit(1)

        private_key = prompt("Enter your private key:")
        # try:
        #     result = (
        #         subprocess.check_output(
        #             [
        #                 "python",
        #                 str(imageRegistryPy_path),
        # "",
        # "",
        # "",
        # str(private_key),
        # "validateAddress",
        #             ]
        #         )
        #         .decode()
        #         .strip()
        #     )
        # except subprocess.CalledProcessError as e:
        #     result = ""
        try:
            result = image_registry.main(
                "", "", "", str(private_key), "validateAddress"
            )
        except Exception as e:
            print(e)
            result = ""

        while result != "OK":
            print(result)
            private_key = prompt(
                "Invalid private key. Please enter a valid private key:"
            )
            # try:
            #     result = (
            #         subprocess.check_output(
            #             [
            #                 "python",
            #                 str(imageRegistryPy_path),
            #                 "",
            #                 "",
            #                 "",
            #                 str(private_key),
            #                 "validateAddress",
            #             ]
            #         )
            #         .decode()
            #         .strip()
            #     )
            # except subprocess.CalledProcessError as e:
            #     result = ""
            try:
                result = image_registry.main(
                    "", "", "", str(private_key), "validateAddress"
                )
            except Exception as e:
                print(e)
                result = ""

        print("Inputted Private key is valid.")
        write_env("PRIVATE_KEY", private_key)
        print()
        print("Checking blockchain for required funds...")
        # try:
        #     result = (
        #         subprocess.check_output(
        #             [
        #                 "python",
        #                 str(imageRegistryPy_path),
        #                 "",
        #                 "",
        #                 "",
        #                 str(private_key),
        #                 "checkBalance",
        #             ]
        #         )
        #         .decode()
        #         .strip()
        #     )
        # except subprocess.CalledProcessError as e:
        #     result = ""
        try:
            result = image_registry.main("", "", "", str(private_key), "checkBalance")
        except Exception as e:
            print(e)
            result = ""

        print(f"Available funds: {result}")

        print()
        print(
            f"Checking if project name is available on {BLOCKCHAIN_NETWORK} network and ownership..."
        )
        # try:
        #     result = (
        #         subprocess.check_output(
        #             [
        #                 "python",
        #                 str(imageRegistryPy_path),
        #                 str(BLOCKCHAIN_NETWORK),
        #                 str(PROJECT_NAME),
        #                 str(VERSION),
        #                 str(private_key),
        #             ],
        #             stderr=subprocess.STDOUT,
        #         )
        #         .decode()
        #         .strip()
        #     )
        # except subprocess.CalledProcessError as e:
        #     print(e)
        #     result = ""
        try:
            result = image_registry.main(
                str(BLOCKCHAIN_NETWORK),
                str(PROJECT_NAME),
                str(VERSION),
                str(private_key),
            )
        except Exception as e:
            print(e)
            result = ""

        print(result)
        if "You are not the account holder of the image" in str(result):
            exit()

        print()
        task_percentage = prompt(
            "Please specify the % of a task which will be transferred to your wallet upon successful execution",
            default_value="10",
        )
        write_env("DEVELOPER_FEE", task_percentage)
    else:
        print(
            "Using PROJECT_NAME, BLOCKCHAIN_NETWORK, PRIVATE_KEY, DEVELOPER_FEE from .env"
        )
        print("Checking blockchain for required funds...")
        # try:
        #     result = (
        #         subprocess.check_output(
        #             [
        #                 "python",
        #                 str(imageRegistryPy_path),
        #                 "",
        #                 "",
        #                 "",
        #                 "",
        #                 "checkBalance",
        #             ]
        #         )
        #         .decode()
        #         .strip()
        #     )
        # except subprocess.CalledProcessError as e:
        #     result = ""
        private_key = PRIVATE_KEY
        try:
            result = image_registry.main("", "", "", str(private_key), "checkBalance")
        except Exception as e:
            print(e)
            result = ""

        print(f"Available funds: {result}")
        print()

        # try:
        #     result = (
        #         subprocess.check_output(
        #             [
        #                 "python",
        #                 str(imageRegistryPy_path),
        #                 "",
        #                 "",
        #                 "",
        #                 str(private_key),
        #                 "validateAddress",
        #             ]
        #         )
        #         .decode()
        #         .strip()
        #     )
        # except subprocess.CalledProcessError as e:
        #     result = ""
        try:
            result = image_registry.main(
                "", "", "", str(private_key), "validateAddress"
            )
        except Exception as e:
            print(e)
            result = ""

        while result != "OK":
            print(result)
            private_key = prompt(
                "Invalid private key. Please enter a valid private key:"
            )
            # try:
            #     result = (
            #         subprocess.check_output(
            #             [
            #                 "python",
            #                 str(imageRegistryPy_path),
            #                 "",
            #                 "",
            #                 "",
            #                 str(private_key),
            #                 "validateAddress",
            #             ]
            #         )
            #         .decode()
            #         .strip()
            #     )
            # except subprocess.CalledProcessError as e:
            #     result = ""
            try:
                result = image_registry.main(
                    "", "", "", str(private_key), "validateAddress"
                )
            except Exception as e:
                print(e)
                result = ""

        write_env("PRIVATE_KEY", private_key)
        print()
        print(
            f"Checking if project name is available on {BLOCKCHAIN_NETWORK} network and ownership..."
        )
        # try:
        #     result = (
        #         subprocess.check_output(
        #             [
        #                 "python",
        #                 str(imageRegistryPy_path),
        #                 str(BLOCKCHAIN_NETWORK),
        #                 str(PROJECT_NAME),
        #                 str(VERSION),
        #                 str(private_key),
        #             ],
        #             stderr=subprocess.STDOUT,
        #         )
        #         .decode()
        #         .strip()
        #     )
        # except subprocess.CalledProcessError as e:
        #     print(e)
        #     result = ""
        try:
            result = image_registry.main(
                str(BLOCKCHAIN_NETWORK),
                str(PROJECT_NAME),
                str(VERSION),
                str(private_key),
            )
        except Exception as e:
            print(e)
            result = ""
        print(result)
        if "You are not the account holder of the image" in str(result):
            exit()

    print()
    if SERVICE_TYPE == "Nodenithy":
        print("Adding prerequisites for Nodenithy...")
        run_script_path = Path(__file__).resolve().parent / "nodenithy" / "run.py"
        try:
            subprocess.run(["python", str(run_script_path)], check=True)
        except subprocess.CalledProcessError:
            print("Error running the Nodenithy run script.")
            sys.exit(1)
    elif SERVICE_TYPE == "Pynithy":
        print("Adding prerequisites for Pynithy...")
        # script_path = Path(__file__).resolve().parent / "pynithy" / "run.py"
        print(f"Running script: runScript")
        # try:
        #     subprocess.run(["python", str(script_path)], check=True)
        #     print("")
        # except subprocess.CalledProcessError:
        #     print("Error running the publish script.")
        #     sys.exit(1)
        import ethernity_cloud_sdk_py.commands.pynithy.runner as runScript

        try:
            runScript.main()
            print("")
        except Exception as e:
            print(f"Error running the publish script: {e}")
            sys.exit(1)
    else:
        print("Something went wrong")
        sys.exit(1)


if __name__ == "__main__":
    main()



import requests  # type: ignore
import argparse
import os, sys, time
import json
from tqdm import tqdm
from requests_toolbelt.multipart.encoder import (
    MultipartEncoder,
    MultipartEncoderMonitor,
)


class IPFSClient:
    def __init__(
        self, api_url: str = "http://ipfs.ethernity.cloud:5001", token: str = ""
    ) -> None:
        self.api_url = api_url
        self.headers = {}
        if token:
            self.headers = {"authorization": token}

    def upload_file(self, file_path: str) -> None:
        add_url = f"{self.api_url}/api/v0/add"

        with open(file_path, "rb") as file:
            files = {"file": file}
            response = requests.post(add_url, files=files, headers=self.headers)

        if response.status_code == 200:
            try:
                response_data = response.json()
                ipfs_hash = response_data["Hash"]
                print(f"Successfully uploaded to IPFS. Hash: {ipfs_hash}")
                return ipfs_hash
            except Exception as e:
                print(f"Failed to upload to IPFS. Error: {e}")
                return None
        else:
            print(f"Failed to upload to IPFS. Status code: {response.status_code}")
            print(response.text)
            return None

    def upload_to_ipfs(self, data: str) -> None:
        add_url = f"{self.api_url}/api/v0/add"
        files = {"file": data}
        response = requests.post(add_url, files=files, headers=self.headers)

        if response.status_code == 200:
            try:
                response_data = response.json()
                ipfs_hash = response_data["Hash"]
                print(f"Successfully uploaded to IPFS. Hash: {ipfs_hash}")
                return ipfs_hash
            except Exception as e:
                print(f"Failed to upload to IPFS. Error: {e}")
                return None
        else:
            print(f"Failed to upload to IPFS. Status code: {response.status_code}")
            print(response.text)
            return None

    def upload_folder_to_ipfs(self, folder_path: str) -> None:
        add_url = f"{self.api_url}/api/v0/add?wrap-with-directory=true&pin=true"
        files = []
        for root, dirs, filenames in os.walk(folder_path):
            for filename in filenames:
                file_path = os.path.join(root, filename).replace("\\", "/")
                files.append(("file", (file_path, open(file_path, "rb"))))

        # Calculate the total size of the files to be uploaded
        total_size = sum(os.path.getsize(file[1][0]) for file in files)

        # Create a progress bar
        with tqdm(
            total=total_size, unit="B", unit_scale=True, desc="Uploading"
        ) as pbar:

            def upload_with_progress(file):
                with open(file[1][0], "rb") as f:
                    while True:
                        chunk = f.read(1024)
                        if not chunk:
                            break
                        yield chunk
                        pbar.update(len(chunk))

            # relative_file_path = os.path.relpath(file_path, folder_path)
            # Create a MultipartEncoder for the files
            encoder = MultipartEncoder(
                fields={
                    file[1][0]: (
                        os.path.relpath(file[1][0], folder_path).replace("\\", "/"),
                        open(file[1][0], "rb"),
                    )
                    for file in files
                }
            )

            # Create a monitor for the encoder to update the progress bar
            monitor = MultipartEncoderMonitor(
                encoder, lambda monitor: pbar.update(monitor.bytes_read - pbar.n)
            )

            # Perform the upload
            # response = requests.post(
            #     add_url,
            #     data=monitor,
            #     headers={**self.headers, "Content-Type": monitor.content_type},
            # )
            response = requests.post(
                add_url,
                data=monitor,
                headers={
                    **self.headers,
                    "Content-Type": monitor.content_type,
                    "Content-Length": str(total_size),
                    "Expect": "100-continue",
                },
            )
        if response.status_code == 200:
            try:
                response_data = json.loads(
                    "[" + response.text.replace("\n", ",")[:-1] + "]"
                )
                for file in response_data:
                    if file["Name"] == "":
                        ipfs_hash = file["Hash"]
                        print(f"Successfully uploaded to IPFS. Hash: {ipfs_hash}")
                        return ipfs_hash
            except Exception as e:
                print(f"Failed to upload to IPFS. Error: {e}")
                return None
        else:
            print(f"Failed to upload to IPFS. Status code: {response.status_code}")
            print(response.text)

    def download_file(
        self, ipfs_hash: str, download_path: str, attempt: int = 0
    ) -> None:
        gateway_url = f"https://ipfs.io/ipfs/{ipfs_hash}"
        response = requests.get(url=gateway_url, timeout=60, headers=self.headers)

        if response.status_code == 200:
            with open(download_path, "wb") as file:
                file.write(response.content)
            print(f"File downloaded successfully to {download_path}")
        else:
            print(
                f"Failed to download from IPFS. Attempt {attempt}. Status code: {response.status_code}. Response text: {response.text}.\n{'Trying again...' if attempt < 6 else ''}"
            )
            if attempt < 6:
                self.download_file(ipfs_hash, download_path, attempt + 1)

    def get_file_content(self, ipfs_hash: str, attempt: int = 0) -> None:
        url = self.api_url
        gateway_url = f"{url}/api/v0/cat?arg={ipfs_hash}"
        response = requests.post(url=gateway_url, timeout=60, headers=self.headers)

        if response.status_code == 200:
            # TODO: use a get encoding function to determine the encoding
            return response.content.decode("utf-8")
        else:
            print(
                f"Failed to get content from IPFS. Attempt {attempt}. Status code: {response.status_code}. Response text: {response.text}.\n{'Trying again...' if attempt < 6 else ''}"
            )
            if attempt < 6:
                self.get_file_content(ipfs_hash, attempt + 1)

        return None


def main(
    host="http://ipfs.ethernity.cloud:5001",
    protocol="http",
    port=5001,
    token="",
    hhash="",
    filePath="",
    folderPath="",
    action="upload",
    output="",
):
    global ipfs_client
    ipfs_client = IPFSClient(host, token)

    if action == "upload":
        if filePath:
            hhash = ipfs_client.upload_file(filePath)
            print(f"{hhash}")
            with open("IPFS_DOCKER_COMPOSE_HASH.ipfs", "w") as f:
                f.write(hhash)

        elif folderPath:
            retry_count = 0
            hhash = None
            while (not hhash or hhash == "Upload failed.") and retry_count < 3:
                try:
                    hhash = ipfs_client.upload_folder_to_ipfs(folderPath)
                    with open("./IPFS_HASH.ipfs", "w") as f:
                        f.write(hhash)
                    print(f"{hhash}")
                except Exception as e:
                    print(f"Error uploading folder: {e}")
                if not hhash or hhash == "Upload failed.":
                    retry_count += 1
                    print(f"Retrying... ({retry_count}/3)")
                    # add a delay here
                    time.sleep(5)
            if not hhash or hhash == "Upload failed.":
                print("Failed to upload folder to IPFS, please try again.")
        else:
            print("Please provide a filePath or folderPath for upload.")
    # elif action == "download":
    #     if filePath:
    #         print(f"Downloading file from IPFS: {hhash}")
    #         get_from_ipfs(hhash, filePath)
    #         print(f"File downloaded. {hhash}")
    #     elif folderPath:
    #         print(f"Downloading folder from IPFS: {hhash}")
    #         download_folder_from_ipfs(hhash, folderPath)
    #         print(f"Folder downloaded. {hhash}")
    #     else:
    #         print("Please provide a filePath or folderPath for download.")
    else:
        print("Please provide a valid action (upload, download).")


# Example usage
if __name__ == "__main__":
    parser = argparse.ArgumentParser(description="IPFS Client")
    parser.add_argument(
        "--host", help="IPFS host", default="http://ipfs.ethernity.cloud:5001"
    )
    parser.add_argument("--protocol", help="Protocol (http or https)", default="http")
    parser.add_argument("--port", help="IPFS port", type=int, default=5001)
    parser.add_argument("--token", help="Authorization token", default="")
    parser.add_argument("--hhash", help="IPFS hash", default="")
    parser.add_argument("--filePath", help="Path to the file", default="")
    parser.add_argument("--folderPath", help="Path to the folder", default="")
    parser.add_argument(
        "--action",
        help="Action to perform (upload, download)",
        required=True,
        default="",
    )
    parser.add_argument("--output", help="Output path for download")

    args = parser.parse_args()

    main(
        host=args.host,
        protocol=args.protocol,
        port=args.port,
        token=args.token,
        hhash=args.hhash,
        filePath=args.filePath,
        folderPath=args.folderPath,
        action=args.action,
        output=args.output,
    )



import os
import sys
import shutil
import subprocess
from pathlib import Path
from dotenv import load_dotenv

# For accessing package resources
try:
    from importlib.resources import path as resources_path
except ImportError:
    # For Python versions < 3.7
    from importlib_resources import path as resources_path  # type: ignore


ECRunner = {
    "etny-pynithy-testnet": [
        "0x02882F03097fE8cD31afbdFbB5D72a498B41112c",
        "0x15D73a742529C3fb11f3FA32EF7f0CC3870ACA31",
        "https://core.bloxberg.org",
        8995,
    ],
    "etny-nodenithy-testnet": [
        "0x02882F03097fE8cD31afbdFbB5D72a498B41112c",
        "0x15D73a742529C3fb11f3FA32EF7f0CC3870ACA31",
        "https://core.bloxberg.org",
        8995,
    ],
    "etny-pynithy": [
        "0x549A6E06BB2084100148D50F51CF77a3436C3Ae7",
        "0x15D73a742529C3fb11f3FA32EF7f0CC3870ACA31",
        "https://core.bloxberg.org",
        8995,
    ],
    "etny-nodenithy": [
        "0x549A6E06BB2084100148D50F51CF77a3436C3Ae7",
        "0x15D73a742529C3fb11f3FA32EF7f0CC3870ACA31",
        "https://core.bloxberg.org",
        8995,
    ],
    "ecld-nodenithy-testnet": [
        "0x4274b1188ABCfa0d864aFdeD86bF9545B020dCDf",
        "0xF7F4eEb3d9a64387F4AcEb6d521b948E6E2fB049",
        "https://rpc-mumbai.matic.today",
        80001,
    ],
    "ecld-pynithy": [
        "0x439945BE73fD86fcC172179021991E96Beff3Cc4",
        "0x689f3806874d3c8A973f419a4eB24e6fBA7E830F",
        "https://polygon-rpc.com",
        137,
    ],
    "ecld-nodenithy": [
        "0x439945BE73fD86fcC172179021991E96Beff3Cc4",
        "0x689f3806874d3c8A973f419a4eB24e6fBA7E830F",
        "https://polygon-rpc.com",
        137,
    ],
}


def write_env(key, value, env_file=".env"):
    """
    Write or update key-value pairs in a .env file in the current working directory.
    """
    env_path = os.path.join(current_dir, env_file)
    if not os.path.exists(env_path):
        with open(env_path, "w") as f:
            f.write(f"{key}={value}\n")
    else:
        replaced = False
        with open(env_path, "r") as f:
            lines = f.readlines()
        with open(env_path, "w") as f:
            for line in lines:
                if line.startswith(f"{key}="):
                    f.write(f"{key}={value}\n")
                    replaced = True
                else:
                    f.write(line)
            if not replaced:
                f.write(f"{key}={value}\n")


def run_command(command, can_pass=False):
    """
    Execute a shell command.
    """
    result = subprocess.run(command, shell=True)
    if result.returncode != 0 and not can_pass:
        print(f"Error executing command: {command}")
        sys.exit(1)


def get_command_output(command):
    """
    Execute a shell command and return its output.
    """
    result = subprocess.run(
        command, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE
    )
    if result.returncode != 0:
        return ""
    return result.stdout.decode("utf-8").strip()


def main():
    load_dotenv()
    global current_dir
    # Set current directory
    current_dir = os.getcwd()
    VERSION = os.getenv("VERSION")
    print(f"Building {VERSION}")
    # Determine template name based on the blockchain network
    BLOCKCHAIN_NETWORK = os.getenv("BLOCKCHAIN_NETWORK", "Bloxberg_Testnet")
    templateName = os.getenv("TRUSTED_ZONE_IMAGE", "etny-pynithy-testnet")
    ENCLAVE_NAME_TRUSTEDZONE = templateName
    isMainnet = False if "testnet" in templateName.lower() else True

    PROJECT_NAME = os.getenv("PROJECT_NAME")

    # Remove the 'registry' directory if it exists
    shutil.rmtree("./registry", ignore_errors=True)

    # Set the build directory path
    build_dir = Path(__file__).resolve().parent / "build"

    # Stop and remove any running Docker containers or images that might conflict
    dockerPS = get_command_output("docker ps --filter name=registry -q")
    if dockerPS:
        run_command(f'docker stop {" ".join(dockerPS.splitlines())}')

    remainingContainers = get_command_output("docker ps --filter 'name=etny' -a -q")
    if remainingContainers:
        run_command(f"docker stop {remainingContainers} -f")

    remainingContainers = get_command_output("docker ps --filter 'name=las' -a -q")
    if remainingContainers:
        run_command(f"docker stop {remainingContainers} -f")

    dockeri = get_command_output("docker ps --filter name=las -q")
    if dockeri:
        run_command(f'docker stop {" ".join(dockeri.splitlines())}')

    dockerRm = get_command_output("docker ps --filter name=registry -q")
    if dockerRm:
        run_command(f'docker rm {" ".join(dockerRm.splitlines())} -f')

    dockerImg = get_command_output('docker images --filter reference="*etny*" -q')
    if dockerImg:
        run_command(f'docker rmi {" ".join(dockerImg.splitlines())} -f')

    dockerImgReg = get_command_output(
        'docker images --filter reference="*registry*" -q'
    )
    if dockerImgReg:
        run_command(f'docker rmi {" ".join(dockerImgReg.splitlines())} -f')

    run_command("docker rm registry -f", can_pass=True)

    # Copy serverless source code to the build directory
    src_dir = "./src/serverless"
    dest_dir = os.path.join(build_dir, "securelock", "src", "serverless")
    print(f"Creating destination directory: {dest_dir}")
    os.makedirs(dest_dir, exist_ok=True)

    print(f"Copying files from {src_dir} to {dest_dir}")
    for file_name in os.listdir(src_dir):
        src_file = os.path.join(src_dir, file_name)
        dest_file = os.path.join(dest_dir, file_name)
        if os.path.isfile(src_file):
            shutil.copy(src_file, dest_file)

    # Change directory to the build directory
    os.chdir(build_dir)

    # Set up Docker registry
    run_command("docker pull registry:2")
    run_command(
        "docker run -d --restart=always -p 5000:5000 --name registry registry:2"
    )

    # Generate the enclave name for securelock
    ENCLAVE_NAME_SECURELOCK = f"{PROJECT_NAME}-SECURELOCK-V3-{BLOCKCHAIN_NETWORK.split('_')[1].lower()}-{VERSION}".replace(
        "/", "_"
    ).replace(
        "-", "_"
    )
    print(f"ENCLAVE_NAME_SECURELOCK: {ENCLAVE_NAME_SECURELOCK}")
    write_env("ENCLAVE_NAME_SECURELOCK", ENCLAVE_NAME_SECURELOCK)

    # Build etny-securelock Docker image
    print("Building etny-securelock")
    os.chdir("securelock")

    # Modify Dockerfile based on the template
    with open("Dockerfile.tmpl", "r") as f:
        dockerfile_secure_template = f.read()

    dockerfile_secure_content = (
        dockerfile_secure_template.replace(
            "__ENCLAVE_NAME_SECURELOCK__", ENCLAVE_NAME_SECURELOCK
        )
        .replace("__BUCKET_NAME__", templateName + "-v3")
        .replace(
            "__SMART_CONTRACT_ADDRESS__",
            ECRunner[templateName][0],
        )
        .replace("__IMAGE_REGISTRY_ADDRESS__", ECRunner[templateName][1])
        .replace("__RPC_URL__", ECRunner[templateName][2])
        .replace("__CHAIN_ID__", str(ECRunner[templateName][3]))
        .replace("__TRUSTED_ZONE_IMAGE__", templateName)
    )
    imagesTag = BLOCKCHAIN_NETWORK.lower()
    if isMainnet:
        imagesTag = BLOCKCHAIN_NETWORK.lower().split("_")[0]
        dockerfile_secure_content.replace(
            "# RUN scone-signer sign", "RUN scone-signer sign"
        )

    with open("Dockerfile", "w") as f:
        f.write(dockerfile_secure_content)

    # Build and push Docker image for etny-securelock
    run_command(
        f"docker build --build-arg ENCLAVE_NAME_SECURELOCK={ENCLAVE_NAME_SECURELOCK} -t etny-securelock:latest ."
    )
    run_command("docker tag etny-securelock localhost:5000/etny-securelock")
    run_command("docker push localhost:5000/etny-securelock")

    # Return to the build directory
    os.chdir("..")

    print(f"ENCLAVE_NAME_TRUSTEDZONE: {ENCLAVE_NAME_TRUSTEDZONE}")
    write_env("ENCLAVE_NAME_TRUSTEDZONE", ENCLAVE_NAME_TRUSTEDZONE)

    # Build etny-trustedzone
    print("Building etny-trustedzone")

    run_command(
        f"docker pull registry.ethernity.cloud:443/debuggingdelight/ethernity-cloud-sdk-registry/ethernity/etny-trustedzone:py_{imagesTag}"
    )
    run_command(
        f"docker tag registry.ethernity.cloud:443/debuggingdelight/ethernity-cloud-sdk-registry/ethernity/etny-trustedzone:py_{imagesTag} localhost:5000/etny-trustedzone"
    )
    run_command("docker push localhost:5000/etny-trustedzone")

    # # Build etny-validator
    # print("Building validator")
    # os.chdir("../validator")
    # run_command("docker build -t etny-validator:latest .")
    # run_command("docker tag etny-validator localhost:5000/etny-validator")
    # run_command("docker push localhost:5000/etny-validator")

    # Build etny-las
    print("Building etny-las")
    run_command(
        f"docker pull registry.ethernity.cloud:443/debuggingdelight/ethernity-cloud-sdk-registry/ethernity/etny-las:py_{imagesTag}"
    )
    run_command(
        f"docker tag registry.ethernity.cloud:443/debuggingdelight/ethernity-cloud-sdk-registry/ethernity/etny-las:py_{imagesTag} localhost:5000/etny-las"
    )
    run_command("docker push localhost:5000/etny-las")

    # Return to the original directory
    os.chdir(current_dir)
    run_command("docker cp registry:/var/lib/registry registry")

    # Clean up
    print("Cleaning up")
    shutil.rmtree(dest_dir, ignore_errors=True)


if __name__ == "__main__":
    main()



class Order:
    def __init__(self, req, order_id):
        self.id = order_id
        self.do_owner = req[0]
        self.dproc = req[1]
        self.do_req = req[2]
        self.dp_req = req[3]
        self.status = req[4]


class MetadataBase:
    def __init__(self, metadata, version):
        self.metadata = metadata
        self._version = version

    @property
    def version(self):
        return self._version

    @property
    def ipfs_hash(self):
        raise NotImplementedError("Subclass must implement this method")

    def checksum(self):
        raise NotImplementedError("Subclass must implement this method")


class PayloadFactory:
    @staticmethod
    def create_payload_metadata(metadata):
        if ':' in metadata:
            if metadata.startswith('v3'):
                return PayloadMetadatav3(metadata)
            else:
                raise ValueError("Invalid payload metadata type")
        else:
            return PayloadMetadataV0(metadata)


class InputFactory:
    @staticmethod
    def create_input_metadata(metadata):
        if ':' in metadata:
            if metadata.startswith('v3'):
                return InputMetadatav3(metadata)
            else:
                raise ValueError("Invalid payload metadata type")
        else:
            return InputMetadataV0(metadata)


class InputMetadataV0(MetadataBase):
    def __init__(self, metadata):
        super().__init__(metadata, 'v0')
        self._ipfs_hash = metadata

    @property
    def ipfs_hash(self):
        return self._ipfs_hash

    @property
    def checksum(self):
        return None


class PayloadMetadataV0(MetadataBase):
    def __init__(self, metadata):
        super().__init__(metadata, 'v0')
        self._ipfs_hash = metadata

    @property
    def ipfs_hash(self):
        return self._ipfs_hash

    @property
    def checksum(self):
        return None


class PayloadMetadatav3(MetadataBase):

    def __init__(self, metadata):
        super().__init__(metadata, 'v3')
        self._checksum = metadata.split(':')[2]
        self._ipfs_hash = metadata.split(':')[1]

    @property
    def ipfs_hash(self):
        return self._ipfs_hash

    @property
    def checksum(self):
        return self._checksum


class InputMetadatav3(MetadataBase):

    def __init__(self, metadata):
        super().__init__(metadata, 'v3')
        self._checksum = metadata.split(':')[2]
        self._ipfs_hash = metadata.split(':')[1]

    @property
    def ipfs_hash(self):
        return self._ipfs_hash

    @property
    def checksum(self):
        return self._checksum if self._checksum != '0' else self._checksum


class DOReqMetadata:
    def __init__(self, req, do_req):
        self._do_req_id = do_req
        self._do_owner = req[0]
        self._metadata1 = req[1]
        self._metadata2 = req[2]
        self._metadata3 = req[3]
        self._metadata4 = req[4]
        self._payload_metadata_obj = PayloadFactory.create_payload_metadata(self.payload_metadata)
        self._input_metadata_obj = InputFactory.create_input_metadata(self.input_metadata)

    @property
    def do_req_id(self):
        return self._do_req_id

    @property
    def do_owner(self):
        return self._do_owner

    @property
    def image_metadata(self):
        return self._metadata1

    @property
    def public_key(self):
        return self.image_metadata.split(':')[5]

    @property
    def image_hash(self):
        return self.image_metadata.split(':')[1]

    @property
    def payload_metadata(self):
        return self._metadata2

    @property
    def payload_metadata_obj(self):
        return self._payload_metadata_obj

    @property
    def input_metadata_obj(self):
        return self._input_metadata_obj

    @property
    def input_metadata(self):
        return self._metadata3

    @property
    def node_address(self):
        return self._metadata4


# add_do_req(..., 'v3:image_:....:...:', 'v3:payload_hash:checksum', 'v3::0', 'node_address')
'''
input + payload
v0: 'ipfs_hash'
v3: 'v3:file_ipfs_hash:file_checksum'

image
v0: 'ipfs_hash:image_name'
v3: 'v3:image_ipfs_hash:image_name:docker_Compose_ipfs_hash:client_challenge_ipfs_hash:client_public_cert'
'''
if __name__ == '__main__':
    obj = PayloadFactory.create_payload_metadata('v3:some_img_hash:fucker')
    obj2 = PayloadFactory.create_payload_metadata('must not!')



def main_init():
    from ethernity_cloud_sdk_py.commands.init import main

    main()


def main_build():
    from ethernity_cloud_sdk_py.commands.build import main

    main()


def main_publish():
    from ethernity_cloud_sdk_py.commands.publish import main

    main()



#!/usr/local/bin/python

import os

from image_registry import ImageRegistry
import re

try:
    print("*********************")

    def extract_certificate(certificate_string):
        pattern = r"(?<=-----BEGIN CERTIFICATE-----\n)(.*?)(?=\n-----END CERTIFICATE-----)"
        matches = re.findall(pattern, certificate_string, re.DOTALL)
        if matches:
            return matches[0].strip()
        else:
            return None


    def read_file_content(file_path):
        with open(file_path, 'r') as file:
            content = file.read()
        return content


    secure_lock = read_file_content('certificate.securelock.crt')
    # secure_lock = extract_certificate(secure_lock)
    print("SECURELOCK:")
    print(secure_lock)

    trusted_zone = read_file_content('certificate.trustedzone.crt')
    # trusted_zone = extract_certificate(trusted_zone)
    print("TRUSTEDZONE:")
    print(trusted_zone)
    ipfs_hash = read_file_content('IPFS_HASH.ipfs')
    ipfs_hash = ipfs_hash.strip()
    ipfs_docker_compose_hash = read_file_content('IPFS_DOCKER_COMPOSE_HASH.ipfs')
    ipfs_docker_compose_hash = ipfs_docker_compose_hash.strip()
    print("IPFS_HASH: " + ipfs_hash)
    imageRegistry = ImageRegistry()
    imageRegistry.add_trusted_zone_cert(trusted_zone, ipfs_hash, 'etny-pynithy-testnet', ipfs_docker_compose_hash, os.getenv("ENCLAVE_NAME_TRUSTEDZONE"), 10)
    imageRegistry.add_secure_lock_and_validate_image_cert(secure_lock, ipfs_hash, 'etny-pynithy-testnet', ipfs_docker_compose_hash, os.getenv("ENCLAVE_NAME_SECURELOCK"), 0)
    print("************************")
except Exception as e:
    print("An exception occurred: " + str(e))



import os

from minio import Minio
from minio.error import S3Error


class SwiftStreamService:

    def __init__(self, endpoint: str, access_key: str, secret_key: str):
        self.client = Minio(endpoint=endpoint,
                            access_key=access_key,
                            secret_key=secret_key,
                            secure=False)

    def create_bucket(self, bucket_name: str) -> (bool, str):
        try:
            if not self.client.bucket_exists(bucket_name):
                self.client.make_bucket(bucket_name)
            else:
                return False, f"Bucket, {bucket_name} already exists!"
        except S3Error as err:
            return False, err

        return True, f"Bucket, {bucket_name} successfully created!"

    def delete_bucket(self, bucket_name: str) -> (bool, str):
        try:
            if self.client.bucket_exists(bucket_name):
                file_objects = self.client.list_objects(bucket_name)
                if file_objects.size() == 0:
                    self.client.remove_bucket(bucket_name)
                else:
                    for file_object in file_objects:
                        self.delete_file(bucket_name, file_object)
                    self.client.remove_bucket(bucket_name)
            else:
                return False, f"Bucket, {bucket_name} does not exists!"
        except S3Error as err:
            return False, err

        return True, f"Bucket, {bucket_name} successfully deleted!"

    def delete_file(self, bucket_name: str, file_name: str) -> (bool, str):
        try:
            self.client.remove_object(bucket_name, file_name)
        except S3Error as err:
            return False, err

        return True, f"File, {file_name} successfully deleted!"

    def delete_files(self, bucket_name: str, list_of_files: list[str]) -> (bool, str):
        try:
            errors = self.client.remove_objects(bucket_name, list_of_files)
            for error in errors:
                print(f"[Error] Error occurred when deleting file: {error}!")
        except S3Error as err:
            return False, err

        return True, f"Files, {list_of_files} successfully deleted!"

    def upload_file(self, bucket_name: str, file_name: str, file_path: str) -> (bool, str):
        try:
            if self.client.bucket_exists(bucket_name):
                self.client.fput_object(bucket_name, file_name, file_path)
            else:
                self.create_bucket(bucket_name)
                self.client.fput_object(bucket_name, file_name, file_path)
        except S3Error as err:
            return False, err

        return True, f"{file_path} is successfully uploaded to bucket {bucket_name}."

    def upload_files(self, bucket_name: str, list_of_files: list[str],
                     upload_file_paths: list[str]) -> (bool, str):
        try:
            if self.client.bucket_exists(bucket_name):
                for file_idx in range(len(upload_file_paths)):
                    self.client.fput_object(bucket_name,
                                            list_of_files[file_idx],
                                            upload_file_paths[file_idx])
            else:
                self.create_bucket(bucket_name)
                for file_idx in range(len(upload_file_paths)):
                    self.client.fput_object(bucket_name,
                                            list_of_files[file_idx],
                                            upload_file_paths[file_idx])
        except S3Error as err:
            return False, err

        return True, f"{upload_file_paths} are successfully uploaded to bucket {bucket_name}."

    def download_file(self, bucket_name: str, file_name: str, file_path: str) -> (bool, str):
        try:
            if self.client.bucket_exists(bucket_name):
                self.client.fget_object(bucket_name, file_name, file_path)
            else:
                return False, f"Bucket, {bucket_name} does not exists!"
        except S3Error as err:
            return False, err

        return True, f"File, {file_name} from bucket {bucket_name} was downloaded in {file_path}."

    def download_files(self, bucket_name: str, list_of_files: list[str],
                       download_file_paths: list[str]) -> (bool, str):
        try:
            if self.client.bucket_exists(bucket_name):
                for file_idx in range(len(list_of_files)):
                    self.client.fget_object(bucket_name,
                                            list_of_files[file_idx],
                                            download_file_paths[file_idx])
            else:
                return False, f"Bucket, {bucket_name} does not exists!"
        except S3Error as err:
            return False, err

        return True, f"{download_file_paths} are successfully uploaded to bucket {bucket_name}."

    def get_file_content_bytes(self, bucket_name: str, file_name: str) -> (bool, bytes):
        response = None
        try:
            response = self.client.get_object(bucket_name, file_name)
            _d = b''
            for data in response.stream(amt=1024 * 1024):
                _d = _d + data

        except S3Error as err:
            return False, err
        finally:
            if response is not None:
                response.close()
                response.release_conn()
        return True, _d

    def get_file_content(self, bucket_name: str, file_name: str) -> (bool, str):
        status, content = self.get_file_content_bytes(bucket_name, file_name)
        if status:
            return True, content.decode('utf-8')
        return status, content

    def put_file_content(self, bucket_name: str, object_name: str, object_path: str,
                         object_data: object = None) -> (bool, str):
        try:
            if object_data is not None:
                self.client.put_object(bucket_name,
                                       object_name,
                                       object_data,
                                       len(object_data.getbuffer()))
            else:
                object_stat = os.stat(object_path)
                with open(object_path, 'rb') as file_data:
                    self.client.put_object(bucket_name,
                                           object_name,
                                           file_data,
                                           object_stat.st_size)
                file_data.close()
        except S3Error as err:
            return False, err

        return True, f"{object_name} is successfully uploaded to bucket {bucket_name}."

    def is_object_in_bucket(self, bucket_name: str, object_name: str) -> (bool, str):
        found = False
        try:
            result = self.client.stat_object(bucket_name, object_name)
            if result:
                found = True
        except S3Error as err:
            return False, err

        if found:
            return found, f"{object_name} exists inside {bucket_name}."
        else:
            return found, f"{object_name} doesn't exists inside {bucket_name}."

    def _list_buckets(self) -> None:
        buckets = self.client.list_buckets()
        for bucket in buckets:
            print(bucket.name, bucket.creation_date)

    def _list_objects(self, bucket_name: str) -> None:
        objects = self.client.list_objects(bucket_name)
        for obj in objects:
            print("-> ", obj.object_name, obj.owner_name)

    def _is_bucket(self, bucket_name: str) -> bool:
        return self.client.bucket_exists(bucket_name)



import os
import sys
from dotenv import load_dotenv

load_dotenv()

from ethernity_cloud_runner_py.runner import EthernityCloudRunner  # type: ignore


def execute_task() -> None:
    ipfs_address = "http://ipfs.ethernity.cloud:5001/api/v0"

    code = 'hello("Hello, Python World!")'

    runner = EthernityCloudRunner()
    runner.initialize_storage(ipfs_address)

    resources = {
        "taskPrice": 8,
        "cpu": 1,
        "memory": 1,
        "storage": 1,
        "bandwidth": 1,
        "duration": 1,
        "validators": 1,
    }
    # this will execute a new task using Python template and will run the code provided above
    # the code will run on the TESTNET network
    runner.run(
        os.getenv("PROJECT_NAME"),
        code,
        "0xd58f5C1834279ABD601df85b3E4b2323aDD4E75e",
        resources,
        os.getenv("ENCLAVE_NAME_TRUSTEDZONE", ""),
    )


if __name__ == "__main__":
    execute_task()



import time
from decimal import Decimal, getcontext

def compute(n):
    getcontext().prec = n
    res = Decimal(0)
    for i in range(n):
        a = Decimal(1)/(16**i)
        b = Decimal(4)/(8*i+1)
        c = Decimal(2)/(8*i+4)
        d = Decimal(1)/(8*i+5)
        e = Decimal(1)/(8*i+6)
        r = a*(b-c-d-e)
        res += r
    return res


t1 = time.time()
res = compute(2000)
dt = time.time()-t1

___etny_result___(str(res))



